; Starfield effect by Petroizki

.686
.model flat, stdcall
	include windows.inc
	
	include gdi32.inc
	include kernel32.inc
	include user32.inc
	
	include pmacros100.inc
	
	AMOUNT_OF_DOTS EQU <512>				; how many dots there are, this cannot be 0
	DOT_SPEED EQU <0.15>
	MINIMUM_LIFETIME EQU <30>				; minimum lifetime the dots have (in frames)
	REFRESH_DELAY EQU <33>					; ms, about 30fps
	SCREEN_HEIGHT EQU <480>					; window height
	SCREEN_WIDTH EQU <640>					; window width
	
	CreateDot proto :DWORD
	CreateDots proto
	main proto
	UpdateDots proto :HDC
	WndProc proto :DWORD, :DWORD, :DWORD, :DWORD
	
	POSITION2D struct
		x REAL4 ?
		y REAL4 ?
	POSITION2D ends
.data?
	dots POSITION2D AMOUNT_OF_DOTS dup (<?>)
.data
	g_dotspeed REAL4 DOT_SPEED
	g_winclass WNDCLASSEX <SIZEOF WNDCLASSEX, CS_HREDRAW or CS_VREDRAW, WndProc, 0, 0, NULL, NULL, NULL, \
		NULL, NULL, OFFSET STRING_WINCLASS, NULL>
	g_world RECT <?>
	g_worldcenter POSITION2D <?>			; center point of world, the point where viewer is going
	
	STRING_WINCLASS db "STARFIELD",0		; class name for createwindowex
	STRING_WINNAME db "Starfield",0			; window title
.code
start:
	fncall main
	fncall ExitProcess, 0

;------------------------------------------------------------------------------
; Program's main proc. Creates window, and handles the main loop.
;------------------------------------------------------------------------------
	LOCALS msg:MSG, temp:DWORD
func main, uses ebx edi esi
	fncall GetStockObject, BLACK_BRUSH		; get background brush
	_mov dword ptr [g_winclass.hbrBackground], eax
	fncall LoadCursor, NULL, IDC_ARROW		; get desktop cursor
	_mov dword ptr [g_winclass.hCursor], eax
	fncall GetModuleHandle, NULL			; get process module handle
	_mov dword ptr [g_winclass.hInstance], eax
	
	fncall RegisterClassEx, ADDR g_winclass	; register windowclass
	test eax, eax
	jz @force_exit							; fuck it
	
	fncall CreateWindowEx, 0, ADDR STRING_WINCLASS, ADDR STRING_WINNAME, WS_MINIMIZEBOX or WS_SYSMENU or WS_VISIBLE, \
		15, 20, SCREEN_WIDTH, SCREEN_HEIGHT, NULL, NULL, g_winclass.hInstance, NULL
	test eax, eax
	jz @force_exit							; terminate prog, it would be nice to have some error messages, but who cares
	
	_mov ebx, eax							; store the window handle
	
	fncall GetDC, eax						; get window DC
	_mov edi, eax
	
	fncall GetClientRect, ebx, ADDR g_world	; get window rectangle
	
	_mov eax, g_world.right					; get right edge
	shr eax, 1								; divide it by 2 to get center point
	_mov dword ptr [LOCAL temp], eax
	fild dword ptr [LOCAL temp]				; load center x-point to fpu register
	fstp dword ptr [g_worldcenter.x]		; store it as fpu
	
	_mov eax, g_world.bottom					; get bottom edge
	shr eax, 1								; divide it by 2 to get center point
	_mov dword ptr [LOCAL temp], eax
	fild dword ptr [LOCAL temp]				; load center y-point to fpu register
	fstp dword ptr [g_worldcenter.y]		; store it as fpu
	
	fncall CreateDots
	
	fncall GetTickCount						; get current time
	lea esi, [eax + REFRESH_DELAY]			; now holds time when screen should be refreshed next
	
@main_loop:
	fncall PeekMessage, ADDR [LOCAL msg], NULL, 0, 0, PM_REMOVE
	test eax, eax
	.IF !zero?
		_cmp dword ptr [LOCAL msg].MSG.message, WM_QUIT
		je @end_main_loop					; quit loop if quit message was encountered
		
		fncall TranslateMessage, ADDR [LOCAL msg]
		fncall DispatchMessage, ADDR [LOCAL msg]
	.ENDIF

	fncall GetTickCount						; get current time
	_cmp esi, eax
	ja @main_loop							; jump if not enough time passed
	lea esi, [eax + REFRESH_DELAY]			; save the next update time
	
	fncall FillRect, edi, ADDR g_world, g_winclass.hbrBackground ; flush background
	fnjmp @main_loop, UpdateDots, edi		; update dot positions and draw dots
@end_main_loop:
	fncall ReleaseDC, ebx, edi				; release dc..
@force_exit:
	ret
endf main

;------------------------------------------------------------------------------
; Windows callback proc.
;------------------------------------------------------------------------------	
func WndProc, hWnd:DWORD, Msg:DWORD, wParam:DWORD, lParam:DWORD
	_cmp dword ptr [PARAM Msg], WM_DESTROY	; check if the program wants to die
	je @WM_DESTROY
	
	jmp DefWindowProc
@WM_DESTROY:
	fncall PostQuitMessage, 0
	xor eax, eax
	ret
endf WndProc

;------------------------------------------------------------------------------	
	LOCALS temp:DWORD
func CreateDot, uses edi, dot:DWORD
	_mov edi, [PARAM dot]
	
	; creates random x position for dot
	fncall Rand, g_world.right
	_mov dword ptr [LOCAL temp], eax
	fild dword ptr [LOCAL temp]
	fstp dword ptr [edi.POSITION2D.x]
	
	; creates random y position for dot
	fncall Rand, g_world.bottom
	_mov dword ptr [LOCAL temp], eax
	fild dword ptr [LOCAL temp]
	fstp dword ptr [edi.POSITION2D.y]

	ret
endf CreateDot

;------------------------------------------------------------------------------
; Creates all dots.
;------------------------------------------------------------------------------	
func CreateDots, uses ebx edi
	_mov ebx, AMOUNT_OF_DOTS
	_mov edi, OFFSET dots
	
	.REPEAT
		fncall CreateDot, edi
		add edi, SIZEOF POSITION2D
		sub ebx, 1
	.UNTIL zero?
	
	ret
endf CreateDots

;------------------------------------------------------------------------------
; Updates dots position and draws them.
;------------------------------------------------------------------------------	
func UpdateDots, uses ebx esi, hDC:HDC
	_mov ebx, AMOUNT_OF_DOTS
	_mov esi, OFFSET dots
	
	.REPEAT
		; below we check that dot is within the window, otherwise it is created again
		fld dword ptr [esi.POSITION2D.x]
		fldz
		fcomip st(0), st(1)
		ja @createdot_popfpu
		
		fild dword ptr [g_world.right]
		fcomip st(0), st(1)
		jb @createdot_popfpu
		
		fld dword ptr [esi.POSITION2D.y]
		fldz
		fcomip st(0), st(1)
		ja @createdot_popfpu
		
		fild dword ptr [g_world.bottom]
		fcomip st(0), st(1)
		jb @createdot_popfpu
		
		ffreep st(1)
		
		push 0FFFFFFh	; SetPixel color
		
		fld dword ptr [esi.POSITION2D.x]			; get dots x pos
		fsub dword ptr [g_worldcenter.x]			; get vectors x element
		
		fld dword ptr [esi.POSITION2D.y]			; get dots y pos
		fsub dword ptr [g_worldcenter.y]			; get vectors y element
		
		push ecx		; SetPixel y position, here we just make some room for the value
		
		fmul dword ptr [g_dotspeed]					; multiply vectors x element with speed
		fadd dword ptr [esi.POSITION2D.y]			; update position
		fst dword ptr [esi.POSITION2D.y]			; store current position for later use
		fistp dword ptr [esp]						; push the position for SetPixel
		
		push ecx		; SetPixel x position
		
		fmul dword ptr [g_dotspeed]					; multiply vectors y element with speed
		fadd dword ptr [esi.POSITION2D.x]			; update position
		fst dword ptr [esi.POSITION2D.x]			; store current position for later use
		fistp dword ptr [esp]						; push the position for SetPixel
		
		push [PARAM hDC + DWORD*3]; push dc handle
		push OFFSET @F	; push the return address
		jmp SetPixel	; draw the pixel
		
	@createdot_popfpu:
		ffreep st(0)
	@createdot:
		fncall CreateDot, esi
@@:
		add esi, SIZEOF POSITION2D						; set pointer to next dot
		sub ebx, 1										; decrese counter
	.UNTIL zero?
	
	ret
endf UpdateDots
end start